/****************************************************************************
** Meta object code from reading C++ file 'deepinauthframework.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/libdde-auth/deepinauthframework.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'deepinauthframework.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19DeepinAuthFrameworkE_t {};
} // unnamed namespace

template <> constexpr inline auto DeepinAuthFramework::qt_create_metaobjectdata<qt_meta_tag_ZN19DeepinAuthFrameworkE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeepinAuthFramework",
        "startupCompleted",
        "",
        "LimitsInfoChanged",
        "SupportedMixAuthFlagsChanged",
        "FramworkStateChanged",
        "SupportedEncryptsChanged",
        "MFAFlagChanged",
        "FuzzyMFAChanged",
        "PromptChanged",
        "FactorsInfoChanged",
        "MFAInfoList",
        "PINLenChanged",
        "AuthStateChanged",
        "SessionCreated",
        "DeviceChanged",
        "TokenAccountMismatch",
        "CreateAuthController",
        "account",
        "AuthCommon::AuthFlags",
        "authType",
        "appType",
        "DestroyAuthController",
        "StartAuthentication",
        "timeout",
        "EndAuthentication",
        "SendTokenToAuth",
        "AuthCommon::AuthType",
        "token",
        "SetAuthQuitFlag",
        "flag",
        "onDeviceChanged",
        "state",
        "AuthQuitFlag",
        "AutoQuit",
        "ManualQuit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'startupCompleted'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'LimitsInfoChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'SupportedMixAuthFlagsChanged'
        QtMocHelpers::SignalData<void(const int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'FramworkStateChanged'
        QtMocHelpers::SignalData<void(const int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'SupportedEncryptsChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'MFAFlagChanged'
        QtMocHelpers::SignalData<void(const bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'FuzzyMFAChanged'
        QtMocHelpers::SignalData<void(const bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'PromptChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'FactorsInfoChanged'
        QtMocHelpers::SignalData<void(const MFAInfoList &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Signal 'PINLenChanged'
        QtMocHelpers::SignalData<void(const int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'AuthStateChanged'
        QtMocHelpers::SignalData<void(const int, const int, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'SessionCreated'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'DeviceChanged'
        QtMocHelpers::SignalData<void(const int, const int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 },
        }}),
        // Signal 'TokenAccountMismatch'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'CreateAuthController'
        QtMocHelpers::SlotData<void(const QString &, const AuthCommon::AuthFlags, const int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { 0x80000000 | 19, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'DestroyAuthController'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'StartAuthentication'
        QtMocHelpers::SlotData<void(const QString &, const AuthCommon::AuthFlags, const int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { 0x80000000 | 19, 20 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'EndAuthentication'
        QtMocHelpers::SlotData<void(const QString &, const AuthCommon::AuthFlags)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'SendTokenToAuth'
        QtMocHelpers::SlotData<void(const QString &, const AuthCommon::AuthType, const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { 0x80000000 | 27, 20 }, { QMetaType::QString, 28 },
        }}),
        // Slot 'SetAuthQuitFlag'
        QtMocHelpers::SlotData<void(const QString &, const int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::Int, 30 },
        }}),
        // Slot 'SetAuthQuitFlag'
        QtMocHelpers::SlotData<void(const QString &)>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'onDeviceChanged'
        QtMocHelpers::SlotData<void(const int, const int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { QMetaType::Int, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AuthQuitFlag'
        QtMocHelpers::EnumData<enum AuthQuitFlag>(33, 33, QMC::EnumFlags{}).add({
            {   34, AuthQuitFlag::AutoQuit },
            {   35, AuthQuitFlag::ManualQuit },
        }),
    };
    return QtMocHelpers::metaObjectData<DeepinAuthFramework, qt_meta_tag_ZN19DeepinAuthFrameworkE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DeepinAuthFramework::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DeepinAuthFrameworkE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DeepinAuthFrameworkE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19DeepinAuthFrameworkE_t>.metaTypes,
    nullptr
} };

void DeepinAuthFramework::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeepinAuthFramework *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->startupCompleted(); break;
        case 1: _t->LimitsInfoChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->SupportedMixAuthFlagsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->FramworkStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->SupportedEncryptsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->MFAFlagChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->FuzzyMFAChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->PromptChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->FactorsInfoChanged((*reinterpret_cast<std::add_pointer_t<MFAInfoList>>(_a[1]))); break;
        case 9: _t->PINLenChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->AuthStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 11: _t->SessionCreated(); break;
        case 12: _t->DeviceChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->TokenAccountMismatch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->CreateAuthController((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthFlags>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 15: _t->DestroyAuthController((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->StartAuthentication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthFlags>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 17: _t->EndAuthentication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthFlags>>(_a[2]))); break;
        case 18: _t->SendTokenToAuth((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 19: _t->SetAuthQuitFlag((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 20: _t->SetAuthQuitFlag((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->onDeviceChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MFAInfoList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)()>(_a, &DeepinAuthFramework::startupCompleted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const QString & )>(_a, &DeepinAuthFramework::LimitsInfoChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const int )>(_a, &DeepinAuthFramework::SupportedMixAuthFlagsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const int )>(_a, &DeepinAuthFramework::FramworkStateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const QString & )>(_a, &DeepinAuthFramework::SupportedEncryptsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const bool )>(_a, &DeepinAuthFramework::MFAFlagChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const bool )>(_a, &DeepinAuthFramework::FuzzyMFAChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const QString & )>(_a, &DeepinAuthFramework::PromptChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const MFAInfoList & )>(_a, &DeepinAuthFramework::FactorsInfoChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const int )>(_a, &DeepinAuthFramework::PINLenChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const int , const int , const QString & )>(_a, &DeepinAuthFramework::AuthStateChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)()>(_a, &DeepinAuthFramework::SessionCreated, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const int , const int )>(_a, &DeepinAuthFramework::DeviceChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeepinAuthFramework::*)(const QString & )>(_a, &DeepinAuthFramework::TokenAccountMismatch, 13))
            return;
    }
}

const QMetaObject *DeepinAuthFramework::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeepinAuthFramework::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DeepinAuthFrameworkE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DeepinAuthFramework::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void DeepinAuthFramework::startupCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DeepinAuthFramework::LimitsInfoChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeepinAuthFramework::SupportedMixAuthFlagsChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DeepinAuthFramework::FramworkStateChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DeepinAuthFramework::SupportedEncryptsChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DeepinAuthFramework::MFAFlagChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void DeepinAuthFramework::FuzzyMFAChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void DeepinAuthFramework::PromptChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void DeepinAuthFramework::FactorsInfoChanged(const MFAInfoList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void DeepinAuthFramework::PINLenChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void DeepinAuthFramework::AuthStateChanged(const int _t1, const int _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3);
}

// SIGNAL 11
void DeepinAuthFramework::SessionCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void DeepinAuthFramework::DeviceChanged(const int _t1, const int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void DeepinAuthFramework::TokenAccountMismatch(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
