/****************************************************************************
** Meta object code from reading C++ file 'auth_widget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/session-widgets/auth_widget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'auth_widget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16SpacerItemBinderE_t {};
} // unnamed namespace

template <> constexpr inline auto SpacerItemBinder::qt_create_metaobjectdata<qt_meta_tag_ZN16SpacerItemBinderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SpacerItemBinder",
        "requestInvalidateLayout",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestInvalidateLayout'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SpacerItemBinder, qt_meta_tag_ZN16SpacerItemBinderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SpacerItemBinder::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SpacerItemBinderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SpacerItemBinderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16SpacerItemBinderE_t>.metaTypes,
    nullptr
} };

void SpacerItemBinder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SpacerItemBinder *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestInvalidateLayout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SpacerItemBinder::*)()>(_a, &SpacerItemBinder::requestInvalidateLayout, 0))
            return;
    }
}

const QMetaObject *SpacerItemBinder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpacerItemBinder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SpacerItemBinderE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DSingleton<SpacerItemBinder>"))
        return static_cast< Dtk::Core::DSingleton<SpacerItemBinder>*>(this);
    return QObject::qt_metacast(_clname);
}

int SpacerItemBinder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void SpacerItemBinder::requestInvalidateLayout()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN10AuthWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto AuthWidget::qt_create_metaobjectdata<qt_meta_tag_ZN10AuthWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AuthWidget",
        "requestCheckSameNameAccount",
        "",
        "account",
        "switchUser",
        "requestCheckAccount",
        "requestSetKeyboardType",
        "key",
        "requestStartAuthentication",
        "AuthCommon::AuthFlags",
        "authType",
        "sendTokenToAuth",
        "AuthCommon::AuthType",
        "token",
        "requestEndAuthentication",
        "authFinished",
        "updateParentLayout",
        "noPasswordLoginChanged",
        "noPassword",
        "updateBlurEffectGeometry",
        "onNoPasswordLoginChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestCheckSameNameAccount'
        QtMocHelpers::SignalData<void(const QString &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'requestCheckSameNameAccount'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestCheckAccount'
        QtMocHelpers::SignalData<void(const QString &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'requestCheckAccount'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestSetKeyboardType'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'requestStartAuthentication'
        QtMocHelpers::SignalData<void(const QString &, const AuthCommon::AuthFlags)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'sendTokenToAuth'
        QtMocHelpers::SignalData<void(const QString &, const AuthCommon::AuthType, const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 12, 10 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'requestEndAuthentication'
        QtMocHelpers::SignalData<void(const QString &, const AuthCommon::AuthFlags)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'authFinished'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateParentLayout'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noPasswordLoginChanged'
        QtMocHelpers::SignalData<void(const QString &, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'updateBlurEffectGeometry'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onNoPasswordLoginChanged'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AuthWidget, qt_meta_tag_ZN10AuthWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AuthWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AuthWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AuthWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10AuthWidgetE_t>.metaTypes,
    nullptr
} };

void AuthWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AuthWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestCheckSameNameAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->requestCheckSameNameAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->requestCheckAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->requestCheckAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->requestSetKeyboardType((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->requestStartAuthentication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthFlags>>(_a[2]))); break;
        case 6: _t->sendTokenToAuth((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 7: _t->requestEndAuthentication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthFlags>>(_a[2]))); break;
        case 8: _t->authFinished(); break;
        case 9: _t->updateParentLayout(); break;
        case 10: _t->noPasswordLoginChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->updateBlurEffectGeometry(); break;
        case 12: _t->onNoPasswordLoginChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)(const QString & , bool )>(_a, &AuthWidget::requestCheckSameNameAccount, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)(const QString & , bool )>(_a, &AuthWidget::requestCheckAccount, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)(const QString & )>(_a, &AuthWidget::requestSetKeyboardType, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)(const QString & , const AuthCommon::AuthFlags )>(_a, &AuthWidget::requestStartAuthentication, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)(const QString & , const AuthCommon::AuthType , const QString & )>(_a, &AuthWidget::sendTokenToAuth, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)(const QString & , const AuthCommon::AuthFlags )>(_a, &AuthWidget::requestEndAuthentication, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)()>(_a, &AuthWidget::authFinished, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)()>(_a, &AuthWidget::updateParentLayout, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthWidget::*)(const QString & , bool )>(_a, &AuthWidget::noPasswordLoginChanged, 10))
            return;
    }
}

const QMetaObject *AuthWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AuthWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AuthWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int AuthWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void AuthWidget::requestCheckSameNameAccount(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 2
void AuthWidget::requestCheckAccount(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 4
void AuthWidget::requestSetKeyboardType(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void AuthWidget::requestStartAuthentication(const QString & _t1, const AuthCommon::AuthFlags _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void AuthWidget::sendTokenToAuth(const QString & _t1, const AuthCommon::AuthType _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}

// SIGNAL 7
void AuthWidget::requestEndAuthentication(const QString & _t1, const AuthCommon::AuthFlags _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void AuthWidget::authFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void AuthWidget::updateParentLayout()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void AuthWidget::noPasswordLoginChanged(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}
QT_WARNING_POP
