/****************************************************************************
** Meta object code from reading C++ file 'authenticate1interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/global_util/dbus/authenticate1interface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'authenticate1interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN36__OrgDeepinDdeAuthenticate1InterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto __OrgDeepinDdeAuthenticate1Interface::qt_create_metaobjectdata<qt_meta_tag_ZN36__OrgDeepinDdeAuthenticate1InterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "__OrgDeepinDdeAuthenticate1Interface",
        "LimitUpdated",
        "",
        "in0",
        "FrameworkStateChanged",
        "value",
        "SupportEncryptsChanged",
        "SupportedFlagsChanged",
        "Authenticate",
        "QDBusPendingReply<QString>",
        "username",
        "authFlags",
        "appType",
        "GetLimits",
        "PreOneKeyLogin",
        "flag",
        "CallQueued",
        "callName",
        "QList<QVariant>",
        "args",
        "onPendingCallFinished",
        "QDBusPendingCallWatcher*",
        "w",
        "onPropertyChanged",
        "propName",
        "QVariant",
        "FrameworkState",
        "SupportEncrypts",
        "SupportedFlags"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'LimitUpdated'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'FrameworkStateChanged'
        QtMocHelpers::SignalData<void(int) const>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'SupportEncryptsChanged'
        QtMocHelpers::SignalData<void(const QString &) const>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'SupportedFlagsChanged'
        QtMocHelpers::SignalData<void(int) const>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'Authenticate'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &, int, int)>(8, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'GetLimits'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'PreOneKeyLogin'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(int)>(14, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'CallQueued'
        QtMocHelpers::SlotData<void(const QString &, const QList<QVariant> &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'onPendingCallFinished'
        QtMocHelpers::SlotData<void(QDBusPendingCallWatcher *)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { 0x80000000 | 25, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'FrameworkState'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'SupportEncrypts'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'SupportedFlags'
        QtMocHelpers::PropertyData<int>(28, QMetaType::Int, QMC::DefaultPropertyFlags, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<__OrgDeepinDdeAuthenticate1Interface, qt_meta_tag_ZN36__OrgDeepinDdeAuthenticate1InterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject __OrgDeepinDdeAuthenticate1Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Core::DDBusExtendedAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36__OrgDeepinDdeAuthenticate1InterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36__OrgDeepinDdeAuthenticate1InterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN36__OrgDeepinDdeAuthenticate1InterfaceE_t>.metaTypes,
    nullptr
} };

void __OrgDeepinDdeAuthenticate1Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<__OrgDeepinDdeAuthenticate1Interface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->LimitUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->FrameworkStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->SupportEncryptsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->SupportedFlagsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: { QDBusPendingReply<QString> _r = _t->Authenticate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<QString> _r = _t->GetLimits((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QString> _r = _t->PreOneKeyLogin((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->CallQueued((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QVariant>>>(_a[2]))); break;
        case 8: _t->onPendingCallFinished((*reinterpret_cast<std::add_pointer_t<QDBusPendingCallWatcher*>>(_a[1]))); break;
        case 9: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (__OrgDeepinDdeAuthenticate1Interface::*)(const QString & )>(_a, &__OrgDeepinDdeAuthenticate1Interface::LimitUpdated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (__OrgDeepinDdeAuthenticate1Interface::*)(int ) const>(_a, &__OrgDeepinDdeAuthenticate1Interface::FrameworkStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (__OrgDeepinDdeAuthenticate1Interface::*)(const QString & ) const>(_a, &__OrgDeepinDdeAuthenticate1Interface::SupportEncryptsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (__OrgDeepinDdeAuthenticate1Interface::*)(int ) const>(_a, &__OrgDeepinDdeAuthenticate1Interface::SupportedFlagsChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->frameworkState(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->supportEncrypts(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->supportedFlags(); break;
        default: break;
        }
    }
}

const QMetaObject *__OrgDeepinDdeAuthenticate1Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *__OrgDeepinDdeAuthenticate1Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36__OrgDeepinDdeAuthenticate1InterfaceE_t>.strings))
        return static_cast<void*>(this);
    return Dtk::Core::DDBusExtendedAbstractInterface::qt_metacast(_clname);
}

int __OrgDeepinDdeAuthenticate1Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Core::DDBusExtendedAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void __OrgDeepinDdeAuthenticate1Interface::LimitUpdated(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void __OrgDeepinDdeAuthenticate1Interface::FrameworkStateChanged(int _t1)const
{
    QMetaObject::activate<void>(const_cast< __OrgDeepinDdeAuthenticate1Interface *>(this), &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void __OrgDeepinDdeAuthenticate1Interface::SupportEncryptsChanged(const QString & _t1)const
{
    QMetaObject::activate<void>(const_cast< __OrgDeepinDdeAuthenticate1Interface *>(this), &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void __OrgDeepinDdeAuthenticate1Interface::SupportedFlagsChanged(int _t1)const
{
    QMetaObject::activate<void>(const_cast< __OrgDeepinDdeAuthenticate1Interface *>(this), &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
