/****************************************************************************
** Meta object code from reading C++ file 'useravatar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/widgets/useravatar.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'useravatar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10UserAvatarE_t {};
} // unnamed namespace

template <> constexpr inline auto UserAvatar::qt_create_metaobjectdata<qt_meta_tag_ZN10UserAvatarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "UserAvatar",
        "mousePress",
        "",
        "requestDelete",
        "userAvatarClicked",
        "alphaChanged",
        "showFinished",
        "hideFinished",
        "borderSelectedColor",
        "QColor",
        "borderColor",
        "borderWidth",
        "AvatarSize",
        "AvatarSmallSize",
        "AvatarNormalSize",
        "AvatarLargeSize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mousePress'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestDelete'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userAvatarClicked'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'alphaChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showFinished'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hideFinished'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'borderSelectedColor'
        QtMocHelpers::PropertyData<QColor>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(10, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'borderWidth'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AvatarSize'
        QtMocHelpers::EnumData<enum AvatarSize>(12, 12, QMC::EnumFlags{}).add({
            {   13, AvatarSize::AvatarSmallSize },
            {   14, AvatarSize::AvatarNormalSize },
            {   15, AvatarSize::AvatarLargeSize },
        }),
    };
    return QtMocHelpers::metaObjectData<UserAvatar, qt_meta_tag_ZN10UserAvatarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject UserAvatar::staticMetaObject = { {
    QMetaObject::SuperData::link<QPushButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10UserAvatarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10UserAvatarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10UserAvatarE_t>.metaTypes,
    nullptr
} };

void UserAvatar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UserAvatar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mousePress(); break;
        case 1: _t->requestDelete(); break;
        case 2: _t->userAvatarClicked(); break;
        case 3: _t->alphaChanged(); break;
        case 4: _t->showFinished(); break;
        case 5: _t->hideFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UserAvatar::*)()>(_a, &UserAvatar::mousePress, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserAvatar::*)()>(_a, &UserAvatar::requestDelete, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserAvatar::*)()>(_a, &UserAvatar::userAvatarClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserAvatar::*)()>(_a, &UserAvatar::alphaChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserAvatar::*)()>(_a, &UserAvatar::showFinished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserAvatar::*)()>(_a, &UserAvatar::hideFinished, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->borderSelectedColor(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->borderWidth(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBorderSelectedColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setBorderWidth(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *UserAvatar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UserAvatar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10UserAvatarE_t>.strings))
        return static_cast<void*>(this);
    return QPushButton::qt_metacast(_clname);
}

int UserAvatar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPushButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void UserAvatar::mousePress()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void UserAvatar::requestDelete()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void UserAvatar::userAvatarClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void UserAvatar::alphaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void UserAvatar::showFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void UserAvatar::hideFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
