/****************************************************************************
** Meta object code from reading C++ file 'auth_password.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/session-widgets/auth_password.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'auth_password.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12AuthPasswordE_t {};
} // unnamed namespace

template <> constexpr inline auto AuthPassword::qt_create_metaobjectdata<qt_meta_tag_ZN12AuthPasswordE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AuthPassword",
        "focusChanged",
        "",
        "lineEditTextChanged",
        "requestChangeFocus",
        "requestShowKeyboardList",
        "resetPasswordMessageVisibleChanged",
        "isVisible",
        "notifyLockedStateChanged",
        "isLocked",
        "requestPluginConfigChanged",
        "LoginPlugin::PluginConfig",
        "PluginConfig",
        "requestHidePlugin",
        "requestPluginAuthToken",
        "accout",
        "token",
        "requestUpdateBlurEffectGeometry",
        "passwordErrorTipsClearChanged",
        "isClear",
        "setResetPasswordMessageVisible",
        "fromResetDialog",
        "updateResetPasswordUI"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'focusChanged'
        QtMocHelpers::SignalData<void(const bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'lineEditTextChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'requestChangeFocus'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestShowKeyboardList'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resetPasswordMessageVisibleChanged'
        QtMocHelpers::SignalData<void(const bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'notifyLockedStateChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'requestPluginConfigChanged'
        QtMocHelpers::SignalData<void(const LoginPlugin::PluginConfig &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'requestHidePlugin'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestPluginAuthToken'
        QtMocHelpers::SignalData<void(const QString, const QString)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::QString, 16 },
        }}),
        // Signal 'requestUpdateBlurEffectGeometry'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'passwordErrorTipsClearChanged'
        QtMocHelpers::SignalData<void(const bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'setResetPasswordMessageVisible'
        QtMocHelpers::SlotData<void(const bool, bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'setResetPasswordMessageVisible'
        QtMocHelpers::SlotData<void(const bool)>(20, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'updateResetPasswordUI'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AuthPassword, qt_meta_tag_ZN12AuthPasswordE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AuthPassword::staticMetaObject = { {
    QMetaObject::SuperData::link<AuthModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AuthPasswordE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AuthPasswordE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12AuthPasswordE_t>.metaTypes,
    nullptr
} };

void AuthPassword::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AuthPassword *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->focusChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->lineEditTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->requestChangeFocus(); break;
        case 3: _t->requestShowKeyboardList(); break;
        case 4: _t->resetPasswordMessageVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->notifyLockedStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->requestPluginConfigChanged((*reinterpret_cast<std::add_pointer_t<LoginPlugin::PluginConfig>>(_a[1]))); break;
        case 7: _t->requestHidePlugin(); break;
        case 8: _t->requestPluginAuthToken((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->requestUpdateBlurEffectGeometry(); break;
        case 10: _t->passwordErrorTipsClearChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setResetPasswordMessageVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->setResetPasswordMessageVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->updateResetPasswordUI(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)(const bool )>(_a, &AuthPassword::focusChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)(const QString & )>(_a, &AuthPassword::lineEditTextChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)()>(_a, &AuthPassword::requestChangeFocus, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)()>(_a, &AuthPassword::requestShowKeyboardList, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)(const bool )>(_a, &AuthPassword::resetPasswordMessageVisibleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)(bool )>(_a, &AuthPassword::notifyLockedStateChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)(const LoginPlugin::PluginConfig & )>(_a, &AuthPassword::requestPluginConfigChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)()>(_a, &AuthPassword::requestHidePlugin, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)(const QString , const QString )>(_a, &AuthPassword::requestPluginAuthToken, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)()>(_a, &AuthPassword::requestUpdateBlurEffectGeometry, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthPassword::*)(const bool )>(_a, &AuthPassword::passwordErrorTipsClearChanged, 10))
            return;
    }
}

const QMetaObject *AuthPassword::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AuthPassword::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AuthPasswordE_t>.strings))
        return static_cast<void*>(this);
    return AuthModule::qt_metacast(_clname);
}

int AuthPassword::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AuthModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void AuthPassword::focusChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AuthPassword::lineEditTextChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AuthPassword::requestChangeFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void AuthPassword::requestShowKeyboardList()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AuthPassword::resetPasswordMessageVisibleChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void AuthPassword::notifyLockedStateChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void AuthPassword::requestPluginConfigChanged(const LoginPlugin::PluginConfig & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void AuthPassword::requestHidePlugin()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void AuthPassword::requestPluginAuthToken(const QString _t1, const QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void AuthPassword::requestUpdateBlurEffectGeometry()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void AuthPassword::passwordErrorTipsClearChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
