/****************************************************************************
** Meta object code from reading C++ file 'loginwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/lightdm-deepin-greeter/loginwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'loginwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11LoginWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto LoginWindow::qt_create_metaobjectdata<qt_meta_tag_ZN11LoginWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LoginWindow",
        "requestSwitchToUser",
        "",
        "std::shared_ptr<User>",
        "user",
        "requestSetKeyboardLayout",
        "layout",
        "requestCreateAuthController",
        "account",
        "requestDestroyAuthController",
        "requestStartAuthentication",
        "AuthCommon::AuthFlags",
        "authType",
        "sendTokenToAuth",
        "AuthCommon::AuthType",
        "token",
        "requestEndAuthentication",
        "authFinished",
        "requestCheckAccount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestSwitchToUser'
        QtMocHelpers::SignalData<void(std::shared_ptr<User>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'requestSetKeyboardLayout'
        QtMocHelpers::SignalData<void(std::shared_ptr<User>, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'requestCreateAuthController'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'requestDestroyAuthController'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'requestStartAuthentication'
        QtMocHelpers::SignalData<void(const QString &, const AuthCommon::AuthFlags)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'sendTokenToAuth'
        QtMocHelpers::SignalData<void(const QString &, const AuthCommon::AuthType, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 14, 2 }, { QMetaType::QString, 15 },
        }}),
        // Signal 'requestEndAuthentication'
        QtMocHelpers::SignalData<void(const QString &, const int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'authFinished'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestCheckAccount'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LoginWindow, qt_meta_tag_ZN11LoginWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LoginWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<FullScreenBackground::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11LoginWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11LoginWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11LoginWindowE_t>.metaTypes,
    nullptr
} };

void LoginWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LoginWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestSwitchToUser((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<User>>>(_a[1]))); break;
        case 1: _t->requestSetKeyboardLayout((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<User>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->requestCreateAuthController((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->requestDestroyAuthController((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->requestStartAuthentication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthFlags>>(_a[2]))); break;
        case 5: _t->sendTokenToAuth((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AuthCommon::AuthType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: _t->requestEndAuthentication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->authFinished(); break;
        case 8: _t->requestCheckAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)(std::shared_ptr<User> )>(_a, &LoginWindow::requestSwitchToUser, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)(std::shared_ptr<User> , const QString & )>(_a, &LoginWindow::requestSetKeyboardLayout, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)(const QString & )>(_a, &LoginWindow::requestCreateAuthController, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)(const QString & )>(_a, &LoginWindow::requestDestroyAuthController, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)(const QString & , const AuthCommon::AuthFlags )>(_a, &LoginWindow::requestStartAuthentication, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)(const QString & , const AuthCommon::AuthType , const QString & )>(_a, &LoginWindow::sendTokenToAuth, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)(const QString & , const int )>(_a, &LoginWindow::requestEndAuthentication, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)()>(_a, &LoginWindow::authFinished, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginWindow::*)(const QString & )>(_a, &LoginWindow::requestCheckAccount, 8))
            return;
    }
}

const QMetaObject *LoginWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoginWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11LoginWindowE_t>.strings))
        return static_cast<void*>(this);
    return FullScreenBackground::qt_metacast(_clname);
}

int LoginWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FullScreenBackground::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void LoginWindow::requestSwitchToUser(std::shared_ptr<User> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void LoginWindow::requestSetKeyboardLayout(std::shared_ptr<User> _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void LoginWindow::requestCreateAuthController(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void LoginWindow::requestDestroyAuthController(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void LoginWindow::requestStartAuthentication(const QString & _t1, const AuthCommon::AuthFlags _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void LoginWindow::sendTokenToAuth(const QString & _t1, const AuthCommon::AuthType _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void LoginWindow::requestEndAuthentication(const QString & _t1, const int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void LoginWindow::authFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void LoginWindow::requestCheckAccount(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
